import  requests
from api_tester.auth import AuthStrategy
from api_tester.factory import APIRequestFactory


class MakeAPICall:
    def __init__(self, base_url, auth_strategy=None):
        self.base_url = base_url
        self.session = requests.Session()

        if auth_strategy and isinstance(auth_strategy, AuthStrategy):
            auth_strategy.apply(self.session)

    def request(self, method, endpoint, **kwargs):
        api_request = APIRequestFactory.create_request(method, self.base_url, endpoint, **kwargs)
        return api_request.send()



import unittest
from api_tester.client import MakeAPICall
from api_tester.auth import TokenAuth

class TestAPIClient(unittest.TestCase):
    def setUp(self):
        token_auth = TokenAuth("test_token")
        self.client = MakeAPICall("https://jsonplaceholder.typicode.com", auth_strategy=token_auth)

    def test_get(self):
        response = self.client.request("GET", "/posts/1")
        self.assertEqual(response.status_code, 200)

    def test_post(self):
        response = self.client.request("POST", "/posts", json={"title": "test", "body": "content", "userId": 1})
        self.assertEqual(response.status_code, 201)

if __name__ == "__main__":
    unittest.main()



import requests




class APIRequest:
    """Base class for API requests."""

    def __init__(self, base_url, endpoint, **kwargs):
        self.url = f'{base_url}{endpoint}'
        self.kwargs = kwargs

    def send(self):
        raise NotImplementedError("Subclasses must implement the send method")



class GetRequest(APIRequest):
    def send(self):
        return requests.get(self.url, **self.kwargs)


class PostRequest(APIRequest):
    def send(self):
        return requests.post(self.url, **self.kwargs)


class PutRequest(APIRequest):
    def send(self):
        return requests.put(self.url, **self.kwargs)

class PatchRequest(APIRequest):
    def send(self):
        return requests.patch(self.url, **self.kwargs)

class DeleteRequest(APIRequest):
    def send(self):
        return requests.delete(self.url, **self.kwargs)



class APIRequestFactory:
    """Factory class to generate API Requests dynamically."""
    request_types = {
        "GET": GetRequest,
        "POST": PostRequest,
        "PUT": PutRequest,
        "PATCH": PatchRequest,
        "DELETE": DeleteRequest,
    }


    def create_request(method, base_url, endpoint, **kwargs):
        request_class = APIRequestFactory.request_types.get(method.upper())
        if not request_class:
            raise ValueError(f"Unsupported HTTP method: {method}")
        return request_class(base_url, endpoint, **kwargs)


from requests.auth import HTTPBasicAuth


class AuthStrategy:
    """Base Authentication Strategy"""

    def apply(self, session):
        pass


class BasicAuth(AuthStrategy):
    def __init__(self, username, password):
        self.auth = HTTPBasicAuth(username, password)

    def apply(self, session):
        session.auth = self.auth



class TokenAuth(AuthStrategy):
    def __init__(self, token):
        self.token = token

    def apply(self, session):
        session.headers.update({"Authorization": f"Token {self.token}"})


class OAuth(AuthStrategy):
    def __init__(self, token):
        self.token = token

    def apply(self, session):
        session.headers.update({"Authorization": f"Bearer {self.token}"})


import argparse
from .client import MakeAPICall


def main():
    parser = argparse.ArgumentParser(description="API Tester CLI Tool")
    parser.add_argument("method", type=str, help="HTTP Method (GET, POST, PUT, PATCH, DELETE)")
    parser.add_argument("url", type=str, help="API Base URL")
    parser.add_argument("endpoint", type=str, help="API Endpoint")
    parser.add_argument("--data", type=str, help="Request Data (JSON format)", default="{}")

    args = parser.parse_args()


    client = MakeAPICall(args.url)
    response = client.request(args.method, args.endpoint, json=eval(args.data))

    print("Response Status: ", response.status_code)
    print("Response Body: ", response.json())


if __name__ == "__main__":
    main()


from setuptools import setup, find_packages


setup(
    name="restquest",
    version="1.1.1",
    packages=find_packages(),
    install_requires=[
        "requests"
    ],
    entry_points={
        "console_scripts": [
            "api-tester=api_tester.cli:main"
        ]
    },
    author="Moataz Fawzy",
    description="A Python package to test API endpoints easily",
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    author_email="your-email@example.com",
    url="https://github.com/Moataz0000/api_tester",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
